@@ -135,6 +135,23 @@ test_that("rethrow", {
   expect_true(cond$`_nframe` < cond$parent$`_nframe`)
 })
 
+test_that("rethrow without call", {
+  h <- function() h2()
+  h2 <- function() throw(new_error("oops"))
+
+  f <- function() g()
+  g <- function() rethrow(h(), new_error("oops2"), call = FALSE)
+
+  cond <- tryCatch(g(), error = function(e) e)
+  expect_s3_class(cond, "rlib_error")
+  expect_null(cond$call)
+  expect_s3_class(cond$parent, "rlib_error")
+  expect_equal(cond$parent$call, quote(h2()))
+  expect_true(is.integer(cond$`_nframe`))
+  expect_true(is.integer(cond$parent$`_nframe`))
+  expect_true(cond$`_nframe` < cond$parent$`_nframe`)
+})
+
 test_that("rethrow_call", {
 
   cond <- tryCatch(
@@ -305,3 +322,75 @@ test_that("error trace from throw() in subprocess", {
   expect_match(cond$trace$messages[[1]], "subprocess failed: .*processx\\.c")
   expect_match(cond$trace$messages[[2]], "@.*processx\\.c")
 })
+
+test_that("trace is not overwritten", {
+  skip_on_cran()
+  withr::local_options(list(rlib_error_always_trace = TRUE))
+  err <- new_error("foobar")
+  err$trace <- "not really"
+
+  err2 <- tryCatch(throw(err), error = function(e) e)
+  expect_identical(err2$trace, "not really")
+})
+
+test_that("error is printed on error", {
+  skip_on_cran()
+
+  sf <- tempfile(fileext = ".R")
+  op <- sub("\\.R$", ".rds", sf)
+  so <- paste0(sf, "out")
+  se <- paste0(sf, "err")
+  on.exit(unlink(c(sf, op, so, se), recursive = TRUE), add = TRUE)
+
+  expr <- substitute({
+    options(rlib_interactive = TRUE)
+    processx::run(basename(tempfile()))
+  })
+
+  cat(deparse(expr), file = sf, sep = "\n")
+
+  callr::rscript(
+    sf,
+    stdout = so,
+    stderr = se,
+    fail_on_status = FALSE,
+    show = FALSE
+  )
+
+  selines <- readLines(se)
+  expect_true(
+    any(grepl("No such file or directory", selines)) ||
+    any(grepl("Command .* not found", selines))
+  )
+  expect_false(any(grepl("Stack trace", selines)))
+})
+
+test_that("trace is printed on error in non-interactive sessions", {
+
+  sf <- tempfile(fileext = ".R")
+  op <- sub("\\.R$", ".rds", sf)
+  so <- paste0(sf, "out")
+  se <- paste0(sf, "err")
+  on.exit(unlink(c(sf, op, so, se), recursive = TRUE), add = TRUE)
+
+  expr <- substitute({
+    processx::run(basename(tempfile()))
+  })
+
+  cat(deparse(expr), file = sf, sep = "\n")
+
+  callr::rscript(
+    sf,
+    stdout = so,
+    stderr = se,
+    fail_on_status = FALSE,
+    show = FALSE
+  )
+
+  selines <- readLines(se)
+  expect_true(
+    any(grepl("No such file or directory", selines)) ||
+      any(grepl("Command .* not found", selines))
+  )
+  expect_true(any(grepl("Stack trace", selines)))
+})
